/* Spiro by Jimmc
 * Copyright 1996 Jim McBeath
 *
 * Permission to use, copy, and distribute Spiro, in source or binary form,
 * for personal non-commercial use is hereby granted without fee, provided
 * that this copyright notice appears in all copies, and that no charge is
 * associated with such copies.
 *
 * Permission to modify Spiro and make derivative works for personal
 * non-commercial use, and to distribute such modified versions, is hereby
 * granted without fee, provided that the users of such a modified version
 * are clearly notified that the work is a modified version of Spiro and
 * not the original Spiro.  As with unmodified distributions, this copyight
 * notice must appear in all modified copies, and no charge may be associated
 * with such copies.
 *
 * The author makes no representations about the suitability of this software
 * for any purpose.  It is provided "as-is", without any express or implied
 * warranty.  In no event will the author be held liable for any damages
 * arising from the use of this software.
 */
/* FactorMath - a class for utility functions dealing with operations
 * that rely on factoring, such as generating primes,
 * least common multiple, and greatest common divisor.
 *
 * Jim McBeath, May 1996
 */

/* This class can be used to calculate LCMs from the command line,
 * for example try "java LeastCommonMultiple 3 4 5" which is 60 */
class LeastCommonMultiple {
	public static void main(String args[]) {
		int a[] = new int[args.length];
		int i;
		for (i=0; i<args.length; i++) {
			a[i] = Integer.parseInt(args[i].trim());
		}
		int result = FactorMath.leastCommonMultiple(a);
		System.out.println("LCM = " + result);
	}
}

/* This class can be used to calculate GCD from the command line */
class GreatestCommonDivisor {
	public static void main(String args[]) {
		int a[] = new int[args.length];
		int i;
		for (i=0; i<args.length; i++) {
			a[i] = Integer.parseInt(args[i].trim());
		}
		int result = FactorMath.greatestCommonDivisor(a);
		System.out.println("GCD = " + result);
	}
}

class IntVector {
	int count;	/* number of items used in the array */
	int alloc;	/* size of the array */
	int a[];	/* the array of ints */

	public IntVector() {
		alloc = 15;	/* pick a number to start with */
		a = new int[alloc];
	}
	public IntVector(String s) {
		/* make an IntVector from a string like "1,2,3" */
		alloc = 15;	/* pick a number to start with */
		a = new int[alloc];
		int cpos = s.indexOf(",");
		while (cpos>0) {
			String s0 = s.substring(0,cpos).trim();
			addInt(Integer.parseInt(s0));
			s = s.substring(cpos+1);
			cpos = s.indexOf(",");
		}
		addInt(Integer.parseInt(s));
	}
	public static String toCommaString(int a[]) {
		StringBuffer s = new StringBuffer();
		int i;
		for (i=0; i<a.length; i++) {
			if (i>0)
				s.append(",");
			s.append(a[i]);
		}
		return s.toString();
	}
	public void addInt(int n) {
		if (count>=alloc) {
			int newalloc = 2*alloc;
			int[] newa = new int[newalloc];
			int i;
			for (i=0; i<count; i++) {
				newa[i] = a[i];
			}
			a = newa;
			alloc = newalloc;
		}
		a[count++] = n;
	}
	public int intAt(int i) {
		return a[i];
	}
	public int size() {
		return count;
	}
	public String toString() {
		StringBuffer s = new StringBuffer();
		s.append("Int[");
		s.append(count);
		s.append("/");
		s.append(alloc);
		s.append("]={");
		int i;
		for (i=0; i<count; i++) {
			if (i>0)
				s.append(",");
			s.append(a[i]);
		}
		s.append("}");
		return s.toString();
	}
	public String toCommaString() {
		StringBuffer s = new StringBuffer();
		int i;
		for (i=0; i<count; i++) {
			if (i>0)
				s.append(",");
			s.append(a[i]);
		}
		return s.toString();
	}
	public int[] getArray() {
		int x[] = new int[count];
		for (int i=0; i<count; i++) {
			x[i] = a[i];
		}
		return x;
	}
}

class FactorMath {
	public static int leastCommonMultiple(int i1, int i2) {
		int a[] = new int[2]; a[0]=i1; a[1]=i2;
		return leastCommonMultiple(a);
	}
	public static int leastCommonMultiple(int i1, int i2, int i3) {
		int a[] = new int[3]; a[0]=i1; a[1]=i2; a[2]=i3;
		return leastCommonMultiple(a);
	}
	public static int leastCommonMultiple(int a[]) {
		int n = a.length;
		Factors f[] = new Factors[n];
		int i;
		for (i=0; i<n; i++) {
			f[i] = new Factors(a[i]);
//System.out.println("Factors of " +a[i]+ " = " +f[i].factors);
		}
		Factors r = f[0];
		for (i=1; i<n; i++) {
			r = r.union(f[i]);
		}
		return r.product();
	}
	public static int greatestCommonDivisor(int a[]) {
		int n = a.length;
		Factors f[] = new Factors[n];
		int i;
		for (i=0; i<n; i++) {
			f[i] = new Factors(a[i]);
		}
		Factors r = f[0];
		for (i=1; i<n; i++) {
			r = r.intersection(f[i]);
		}
		return r.product();
	}
}

/* Holds a list of factors of an integer */
class Factors {
	IntVector factors;		/* the list of prime factors */
			/* does not include 1, but does include itself */
	public Factors() {
		factors = new IntVector();
		/* return an empty Factors, suitable for building */
	}
	public Factors(int n) {
		factors = new IntVector();
		/* calculate prime factors of n, store into "factors" */
		int i = 0;
		PrimeNumbers.generate(1+(int)Math.sqrt((double)n));
		int p = PrimeNumbers.nth(i);
		int nn = n;
		while (p<=nn) {
			if (nn%p==0) {
				addFactor(p);
				nn = nn/p;
			} else {
				i++;
				p = PrimeNumbers.nth(i);
			}
		}
		if (factors.size()==0)
			addFactor(n);	/* prime - put in itself as factor */
	}
	public void addFactor(int prime) {
		/* Add a new factor to the list.  Assumes it is being
		 * properly added in sorted order. */
		factors.addInt(prime);
	}
	public boolean isPrime() {
		return (factors.size()==1);
			/* if only one factor (itself), then it's prime */
	}
	public int product() {
		/* Multiplies out all of the factors and returns that number */
		int n = factors.size();
		int prod = 1;
		int i;
		for (i=0; i<n; i++) {
			prod *= factors.intAt(i);
		}
		return prod;
	}
	public Factors union(Factors that) {
		/* Takes the union of two lists of factors */
		int thislen = factors.size();
		int thatlen = that.factors.size();
		int thisi = 0;
		int thati = 0;
		Factors result = new Factors();
		while (thisi<thislen && thati<thatlen) {
			int thisprime = factors.intAt(thisi);
			int thatprime = that.factors.intAt(thati);
			if (thisprime>thatprime) {
				result.addFactor(thatprime);
				thati++;
			} else if (thisprime<thatprime) {
				result.addFactor(thisprime);
				thisi++;
			} else /* thisprime==thatprime */ {
				result.addFactor(thisprime);
				thisi++;
				thati++;
			}
		}
		while (thisi<thislen) {
			int thisprime = factors.intAt(thisi);
			result.addFactor(thisprime);
			thisi++;
		}
		while (thati<thatlen) {
			int thatprime = that.factors.intAt(thati);
			result.addFactor(thatprime);
			thati++;
		}
		return result;
	}
	public Factors intersection(Factors that) {
		/* Takes the intersection of two lists of factors */
		int thislen = factors.size();
		int thatlen = that.factors.size();
		int thisi = 0;
		int thati = 0;
		Factors result = new Factors();
		while (thisi<thislen && thati<thatlen) {
			int thisprime = factors.intAt(thisi);
			int thatprime = that.factors.intAt(thati);
			if (thisprime>thatprime) {
				thati++;
			} else if (thisprime<thatprime) {
				thisi++;
			} else /* thisprime==thatprime */ {
				result.addFactor(thisprime);
				thisi++;
				thati++;
			}
		}
		return result;
	}
}

/* PrimeNumbers - a class to generate and store a list of prime numbers */
class PrimeNumbers {
	static IntVector primes;
	static int largestPrime; //that we have yet generated
	public static void generate(int n) {
		/* Generate prime numbers up to n, if not yet done */
		if (largestPrime==0) { /* not yet initialized */
			primes = new IntVector();
			primes.addInt(2);
			largestPrime = 2;
		}
		while (largestPrime<n) {
			checkPrime(++largestPrime);
		}
	}
	private static void checkPrime(int p) {
		/* Check to see if a number is prime; if so, add to list */
		int i;
		for (i=0; i<primes.size(); i++) {
			int d = primes.intAt(i);
			if (p%d==0)	/* p is divisible by d */
				return;	/* not prime */
		}
		primes.addInt(p);
	}
	public static int nth(int n) {
		while (n>=primes.size()) {
			checkPrime(++largestPrime);
		}
		int p = primes.intAt(n);
		return p;
	}
}

/* end */
